package Pastel::GradientPaint;
@ISA = qw(Pastel::Mixin::Mixin);
use strict;
use Carp;

=head2 NAME

Pastel::GradientPaint

=head2 DESCRIPTION

Represent a gradient fill for a shape. Unlike Java 2D graphics SVG supports
two different types of gradient fill- radial and linear. By default the 
constructor creates a linera gradient. To fill a shape with a gradient fill
first create a Pastel::GradientPaint object. For simple linear Gradient with
two colors 
=cut

{ 
    my $id = 0;
    sub _id { return $id++ ; }
}


use constant USER_SPACE => "userSpaceOnUse";
use constant BOUNDING_BOX => "objectBoundingBox";
use constant PAD => "pad";
use constant REFLECT => "reflect";
use constant REPEAT => "repeat";
use constant RADIAL => "radial";
use constant LINEAR => "linear";

=head2 CONTRUCTOR

=head3 Pastel::GradientPaint()

An empty constructor. By default creates a lineargradient.

=head3 Pastel::GradientPaint( -point1=>$point1, -color1=>$color1, -point2=>$point2, -color2=>$color2)

$point1 and $point2 are Pastel::Geometry::Point object and $color1 and 
$color2 are Pastel::Color objects. Creates a linear gradient from $point1
to $point2, starting with $color1 ending in $color2.

=head3 Pastel::GradientPaint(-shape=>Pastel::GradientPaint::RADIAL/LINEAR)

Creates a GradientPaint object with the specified constants: RADIAL or LINEAR.
The constants are specified as: Pastel::GRradientPaint::RADIAL and
Pastel::GradientPaint::LINEAR. You need to specify the stop offest separately.

=head3 Pastel::GradientPaint(-color1=>$color1, -color2=>$color2)

Creates a simple linear horizontal gradient. No GradientStop object necessary.


sub new {
    my $class = shift;
    my $self = {};
    bless $self, ref($class) || $class;
    $self->_init(@_);
    return $self;
}

sub _init {
     my ($self, @args) = @_;
     $self->{id} = $self->get_id();
     return $self;
}

sub get_id {
    return $_[0]->_id();
}
1;
