#! /usr/bin/perl -w

use blib;
use Pastel;
use Pastel::Geometry::GeneralPath;
use Pastel::Geometry::PathIteratorI;
use Test::More 'no_plan';

my $path = Pastel::Geometry::GeneralPath->new();
ok (defined ($path));

is ($path->get_winding_rule, "WIND_NON_ZERO");
is ($path->get_winding_rule, Pastel::Geometry::GeneralPath::WIND_NON_ZERO);

$path->set_winding_rule('WIND_EVEN_ODD');
is ($path->get_winding_rule, 'WIND_EVEN_ODD');

$path->move_to(10, 12);
is ($path->{types}[0], 'SEG_MOVETO');

is($path->{numtypes}, 1);
is($path->{numcoord}, 2);

$path->line_to(15,30);

is($path->{types}[1], 'SEG_LINETO');
is($path->{numtypes}, 2);
is($path->{numcoord}, 4);

$path->quad_to(25,30,35,40);

is($path->{types}[2], 'SEG_QUADTO');
is($path->{numtypes}, 3);
is ($path->{numcoord}, 8);

$path->curve_to(40,45,50,55,60,65);
is($path->{types}[3], 'SEG_CUBICTO');
is($path->{numtypes}, 4);
is ($path->{numcoord}, 14);

my $point = $path->get_current_point();
is ($point->get_x(), 60);
is ($point->get_y(), 65);


$path->close_path();
is($path->{types}[4], 'SEG_CLOSE');
is($path->{numtypes}, 5);
is($path->{numcoord}, 14);

$point = $path->get_current_point();
is ($point->get_x(), 10);
is ($point->get_y(), 12);


my $path2 = Pastel::Geometry::GeneralPath->new(-rule=>'WIND_EVEN_ODD');
is ($path2->get_winding_rule(),'WIND_EVEN_ODD'); 
is ($path2->get_fill_rule(), 'evenodd');

my $g = Pastel::Graphics->new();
my $color = Pastel::Color->red();
$g->set_paint($color);

$path->line_to(60, 75);
$path->line_to(80, 75);

is($path->to_svg(), "M10 12L15 30Q25 30 35 40C40 45 50 55 60 65zV75H80");



